"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createSystem;

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _css = _interopRequireDefault(require("./css"));

var _keyframes = _interopRequireDefault(require("./keyframes"));

var _utils = require("./utils");

// prettier-ignore
var META = ['$wrap', '$pseudo', '$children'];

function getName() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      displayName = _ref.displayName,
      As = _ref.as;

  var asDName = As && As.displayName;
  var asName = As && As.name;
  var tag = (0, _utils.isStr)(As) && As;
  var name = (0, _utils.capitalize)(displayName || asDName || asName || tag || 'div');
  return name[0] === 'Σ' ? name : 'Σ' + name;
}

function separateStyle(props, rules) {
  var style = {};
  var other = {};
  (0, _utils.forEach)(props, function (value, prop) {
    if (rules[prop] || META.includes(prop)) {
      style[prop] = value;
    } else {
      other[prop] = value;
    }
  });
  return [style, other];
} // execute the functional default props by passing them current props


function applyPropsToDefaults(defaults, props) {
  var computedDefaults = {};
  (0, _utils.forEach)(defaults, function (value, prop) {
    var isValFn = (0, _utils.isFn)(value);

    if (prop === 'as' && isValFn) {
      var res = value(props);
      computedDefaults[prop] = (0, _utils.isFn)(res) || (0, _utils.isStr)(res) ? res : value;
    } else if (isValFn) {
      computedDefaults[prop] = value(props);
    } else {
      computedDefaults[prop] = value;
    }
  });
  return computedDefaults;
} // for each style prop, check if the matching rule is a function and execute it


function applyRulesToStyle(style, props, rules, omit) {
  var computedProps = {};
  (0, _utils.forEach)(style, function (value, prop) {
    if (prop === omit) {
      return computedProps[prop] = value;
    }

    var rule = rules[prop];
    var isRuleFn = (0, _utils.isFn)(rule);
    var result = isRuleFn ? rule(value, props) : value;
    var isResultObj = (0, _utils.isObj)(result);

    if (isResultObj) {
      // apply deeper but ignore self referencing rules
      result = applyRulesToStyle(result, props, rules, prop);
    }

    if (isRuleFn && isResultObj) {
      computedProps = (0, _utils.merge)(computedProps, result);
    } else {
      computedProps[prop] = result;
    }
  });
  return computedProps;
}

function getMetaArgs(type, meta, pseudo, children, wrap) {
  switch (type) {
    case '$wrap':
      return [pseudo, children, meta];

    case '$pseudo':
      return [meta, children, wrap];

    case '$children':
      return [pseudo, meta, wrap];
  }
}

function getClassNames(style, rules, pseudo, children, wrap) {
  var computedStyle = {};
  var classNames = [];
  (0, _utils.forEach)(style, function (value, prop) {
    // for meta rules go deeper except inside children rules
    if (META.includes(prop) && (0, _utils.isObj)(value) && !Boolean(children)) {
      (0, _utils.forEach)(value, function (metaStyle, meta) {
        var metaArgs = getMetaArgs(prop, meta, pseudo, children, wrap);
        var metaClassNames = getClassNames.apply(void 0, [metaStyle, rules].concat((0, _toConsumableArray2.default)(metaArgs)));
        classNames.push.apply(classNames, (0, _toConsumableArray2.default)(metaClassNames));
      });
    } else if ((0, _utils.isDef)(value)) {
      var rule = (0, _utils.isStr)(rules[prop]) ? rules[prop] : prop;
      computedStyle[(0, _utils.kebabCase)(rule)] = value;
    }
  });
  return (0, _css.default)(computedStyle, pseudo, children, wrap).concat(classNames);
}

function buildAnimation(steps, props, rules) {
  var computedSteps = {};
  (0, _utils.forEach)(steps, function (style, step) {
    computedSteps[step] = {};
    var computedProps = applyRulesToStyle(style, props, rules);
    (0, _utils.forEach)(computedProps, function (value, prop) {
      var rule = (0, _utils.isStr)(rules[prop]) ? rules[prop] : prop;
      computedSteps[step][(0, _utils.kebabCase)(rule)] = value;
    });
  });
  return (0, _keyframes.default)(computedSteps);
}

function createSystem(renderer) {
  return function system() {
    var rules = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    function createComponent() {
      var defaults = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var wrap = arguments.length > 1 ? arguments[1] : undefined;
      var uniqueClassName = (0, _utils.uid)('c');

      function applySystem(props) {
        var computedDefaults = applyPropsToDefaults(defaults, props);
        var allProps = (0, _utils.merge)(computedDefaults, props);

        var _separateStyle = separateStyle(allProps, rules),
            _separateStyle2 = (0, _slicedToArray2.default)(_separateStyle, 2),
            styleProps = _separateStyle2[0],
            passedProps = _separateStyle2[1];

        var computedProps = applyRulesToStyle(styleProps, allProps, rules);

        var _separateStyle3 = separateStyle(computedProps, rules),
            _separateStyle4 = (0, _slicedToArray2.default)(_separateStyle3, 2),
            style = _separateStyle4[0],
            ruleProps = _separateStyle4[1];

        var computedClassNames = getClassNames(style, rules);
        var classNames = [uniqueClassName].concat(computedClassNames);

        var _merge = (0, _utils.merge)(ruleProps, passedProps),
            Rendered = _merge.as,
            nextProps = (0, _objectWithoutProperties2.default)(_merge, ["as"]);

        return [Rendered || 'div', classNames, nextProps];
      }

      var System = renderer(applySystem);
      System = wrap ? wrap(System) : System;

      System.with = function (props) {
        return createComponent((0, _utils.merge)(defaults, props), wrap);
      };

      System.compose = function () {
        for (var _len = arguments.length, next = new Array(_len), _key = 0; _key < _len; _key++) {
          next[_key] = arguments[_key];
        }

        return createComponent(defaults, _utils.compose.apply(void 0, [wrap].concat(next)));
      }; // prettier-ignore


      System.system = function (extra) {
        return system((0, _utils.merge)(rules, extra)).compose(wrap).with(defaults);
      }; // prettier-ignore


      System.toString = function () {
        return '.' + uniqueClassName;
      };

      System.displayName = getName(defaults);

      System.animate = function (timing, steps) {
        var computedAnim = buildAnimation(steps, defaults, rules);
        var animation = (defaults.animation ? defaults.animation + ', ' : '') + computedAnim + ' ' + timing; // prettier-ignore

        var Animated = createComponent((0, _utils.merge)(defaults, {
          animation: animation
        }), wrap);
        Animated.animationName = computedAnim;
        return Animated;
      };

      return System;
    }

    return createComponent();
  };
}
//# sourceMappingURL=system.js.map